@@ -2,6 +2,8 @@ import asyncio
 import hashlib
 import inspect
 import logging.config
+import mimetypes
+import sys
 from enum import Enum, auto
 from functools import wraps, partial
 from pathlib import Path
@@ -17,6 +19,21 @@ from .config.settings import *
 from .login import Session, Response
 from .utils import find_key
 
+try:
+    if get_ipython().__class__.__name__ == 'ZMQInteractiveShell':
+        import nest_asyncio
+
+        nest_asyncio.apply()
+except:
+    ...
+
+if sys.platform != 'win32':
+    import uvloop
+
+    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+else:
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
+
 logging.config.dictConfig(log_config)
 logger = logging.getLogger(__name__)
 
@@ -146,8 +163,8 @@ async def upload_media(fname: str, auth_session: Session, is_dm=False):
         file = Path(fname)
         total_bytes = file.stat().st_size
         upload_type = 'dm' if is_dm else 'tweet'
-        media_type = MEDIA[file.suffix]['type']
-        media_category = MEDIA[file.suffix]['category'][upload_type]
+        media_type = mimetypes.guess_type(file)[0]
+        media_category = f'{upload_type}_{media_type.split("/")[0]}'
 
         if media_category in {'dm_image', 'tweet_image'} and total_bytes > MAX_IMAGE_SIZE:
             raise Exception(f'Image too large: max is {(MAX_IMAGE_SIZE / 1e6):.2f} MB')
@@ -288,6 +305,7 @@ def unretweet(tweet_id: int, session: Session) -> Response:
     return graphql_request(tweet_id, Operation.DeleteRetweet.name, 'source_tweet_id', session)
 
 
+@log(level=logging.DEBUG, info=['json'])
 def get_tweets(user_id: int, session: Session) -> Response:
     operation = Operation.UserTweets.name
     qid = operations[operation]['queryId']
@@ -352,13 +370,10 @@ def unblock(user_id: int, session: Session) -> Response:
 
 
 @log(level=logging.DEBUG, info=['text'])
-def update_search_settings(session: Session, **kwargs) -> Response:
-    if kwargs.get('incognito'):
-        kwargs.pop('incognito')
-        settings = account_search_settings.copy()
-    else:
-        settings = {}
-    settings |= kwargs
+def update_search_settings(session: Session, hide_blocked=False, hide_nsfw=False) -> Response:
+    settings = account_search_settings.copy()
+    settings['optInFiltering'] = hide_nsfw
+    settings['optInBlocking'] = hide_blocked
     twid = int(session.cookies.get_dict()['twid'].split('=')[-1].strip('"'))
     headers = get_auth_headers(session=session)
     r = session.post(
@@ -378,13 +393,8 @@ def update_content_settings(session: Session, **kwargs) -> Response:
     @param kwargs: settings to enable/disable
     @return: updated settings
     """
-    if kwargs.get('incognito'):
-        kwargs.pop('incognito')
-        settings = content_settings.copy()
-    else:
-        settings = {}
-    settings |= kwargs
-    return api_request(settings, 'account/settings.json', session)
+    kwargs |= content_settings
+    return api_request(kwargs, 'account/settings.json', session)
 
 
 def build_query(params: dict) -> str:
@@ -405,11 +415,17 @@ def stats(rest_id: int, session: Session) -> Response:
 
 
 @log(level=logging.DEBUG, info=['json'])
-def dm(text: str, sender: int, receiver: int, session: Session, filename: str = '') -> Response:
+def dm(text: str, receivers: list[int], session: Session, filename: str = '') -> Response:
     operation = Operation.useSendMessageMutation.name
     qid = operations[operation]['queryId']
     params = operations[operation]
-    params['variables']['target']['conversation_id'] = '-'.join(map(str, sorted((sender, receiver))))
+
+    ## todo single DM request not needed, `participant_ids` field handles single DM request
+    # if len(receivers) == 1:
+    #     params['variables']['target'] = {'conversation_id': '-'.join(map(str, sorted((sender, *receivers))))}
+    # else:
+    params['variables']['target'] = {"participant_ids": receivers}
+
     params['variables']['requestId'] = str(uuid1(getnode()))  # can be anything
     url = f"https://api.twitter.com/graphql/{qid}/{operation}"
     if filename:
